@@ -1,20 +1,33 @@
 from datetime import datetime
 from typing import Any, List, Tuple
 
+from uplink import (
+    Body,
+    Query,
+    delete,
+    get,
+    json,
+    params,
+    post,
+    put,
+    response_handler,
+    returns,
+)
+
 from bpkio_api.caching import cache_api_results
 from bpkio_api.consumer import BpkioSdkConsumer
 from bpkio_api.exceptions import BroadpeakIoHelperError, ResourceExistsError
 from bpkio_api.helpers.list import collect_from_ids, get_all_with_pagination
 from bpkio_api.helpers.objects import find_duplicates_of
 from bpkio_api.helpers.search import SearchMethod, search_array_with_filters
-from bpkio_api.models import ContentReplacementSlot
+from bpkio_api.helpers.upsert import UpsertOperationType, upsert_status
+from bpkio_api.models import (
+    ContentReplacementSlot,
+    VirtualChannelSlot,
+    VirtualChannelSlotIn,
+)
 from bpkio_api.models import Services as Svc
-from bpkio_api.models import VirtualChannelSlot, VirtualChannelSlotIn
-from bpkio_api.response_handler import postprocess_response, return_count
-from uplink import (Body, Query, delete, get, json, params, post, put,
-                    response_handler, returns)
-
-from .enums import UpsertOperationType
+from bpkio_api.response_handler import extract_count, postprocess_response
 
 
 @response_handler(postprocess_response)
@@ -57,16 +70,21 @@ class ServicesApi(BpkioSdkConsumer):
                     "not recognised as a valid service type"
                 )
 
+    @response_handler(extract_count)
+    @get("services")
+    def count(self) -> int:
+        """Get a count of all services"""
+
     @returns.json(List[Svc.ServiceSparse])
     @get("services")
-    def get_page(self, offset: Query = 0, limit: Query = 5) -> List[Svc.ServiceSparse]:  # type: ignore
+    def _get_page(self, offset: Query = 0, limit: Query = 5) -> List[Svc.ServiceSparse]:  # type: ignore
         """List all services"""
 
     # === Helpers ===
 
     @cache_api_results("list_services")
     def list(self):
-        return get_all_with_pagination(self.get_page)
+        return get_all_with_pagination(self._get_page, count_fn=self.count)
 
     def search_by_type(self, type: Svc.ServiceType) -> List[Svc.ServiceSparse]:
         all_items = self.list()
@@ -122,14 +140,11 @@ class ServicesApi(BpkioSdkConsumer):
         endpoint.
 
         Args:
-            service_id (int): The service identifier
-
-        Raises:
-            e: _description_
+            service_id: The service identifier
 
         Returns:
-            VirtualChannelService, AdInsertionService, ContentReplacementService :
-              A specific sub-type of service
+            VirtualChannelService, AdInsertionService, ContentReplacementService, AdaptiveStreamingCdnService:
+            A specific sub-type of service
         """
 
         candidates = self.search(
@@ -199,37 +214,57 @@ class ServicesApi(BpkioSdkConsumer):
         service: Svc.ServiceIn,
         if_exists: str = "retrieve",
         unique_fields: List[str | Tuple] = [],
-    ) -> Tuple[
+    ) -> (
         Svc.VirtualChannelService
         | Svc.ContentReplacementService
         | Svc.AdInsertionService
-        | Svc.ServiceIn,
-        UpsertOperationType,
-    ]:
-        """Creates a service with adaptable behaviour if it already exist.
+        | Svc.ServiceIn
+    ):
+        """Create, retrieve, or update a service based on existence.
+
+        This method provides flexible handling of service creation with different
+        behaviors when the service already exists.
 
         Args:
-            service (ServiceIn): The payload for the service to create
-            if_exists (str): What action to take if it exists:
-              `error` (default) returns an error;
-              `retrieve` returns the existing object;
-              `update` updates the existing object.
-            unique_fields (List[str | Tuple], optional): List of the fields
-            or combination of fields to check for unicity. Defaults to [].
+            service: The service data to create or update.
+            if_exists: Controls behavior when a service with the same name exists:
+
+                - `error`: Raise an error (default)
+                - `retrieve`: Return the existing service
+                - `update`: Update the existing service with new data
 
         Returns:
-            Tuple[VirtualChannelService | ContentReplacementService |
-            AdInsertionService | ServiceIn, int]:
-            The resource created or retrieved, with an indicator:
-            1 = created, 0 = retrieved, 2 = updated, -1 = failed
+            The Service object (created, retrieved, or updated)
 
+        Raises:
+            ResourceExistsError: If the service exists and if_exists="error".
+            ValidationError: If the service data is invalid.
+
+        Examples:
+            ```python
+            from bpkio_api.helpers.upsert import UpsertOperationType, upsert_status
+            
+            # Create or update
+            service = ContentReplacementServiceIn(name="News", description="News content")
+            result = api.upsert(service, if_exists="update")            
+            
+            # Optional: Check the status of the operation
+            status = upsert_status.get()
+            if status == UpsertOperationType.CREATED:
+                print("Created new service")
+            elif status == UpsertOperationType.UPDATED:
+                print("Updated existing service")
+            ```
         """
 
         try:
-            return (self.create(service), UpsertOperationType.CREATED)
+            created_resource = self.create(service)
+            upsert_status.set(UpsertOperationType.CREATED)
+            return created_resource
         except ResourceExistsError as e:
             if if_exists == "error":
-                return (service, UpsertOperationType.ERROR)
+                upsert_status.set(UpsertOperationType.ERROR)
+                return service
 
             unique_fields = list(set(unique_fields + ["name"]))
             for fld in unique_fields:
@@ -245,10 +280,12 @@ class ServicesApi(BpkioSdkConsumer):
                     existing_resource = self.retrieve(dupes[0][1].id)
 
                     if if_exists == "retrieve":
-                        return (existing_resource, UpsertOperationType.RETRIEVED)
+                        upsert_status.set(UpsertOperationType.RETRIEVED)
+                        return existing_resource
                     elif if_exists == "update":
                         updated_resource = self._update(existing_resource.id, service)
-                        return (updated_resource, UpsertOperationType.UPDATED)
+                        upsert_status.set(UpsertOperationType.UPDATED)
+                        return updated_resource
 
     def pause(
         self, service_id: int
@@ -297,14 +334,17 @@ class ContentReplacementServiceApi(BpkioSdkConsumer):
     @returns.json(Svc.ContentReplacementService)
     @post("services/content-replacement")
     def create(
-        self, service: Body(type=Svc.ContentReplacementServiceIn)  # type: ignore
+        self,
+        service: Body(type=Svc.ContentReplacementServiceIn),  # type: ignore
     ) -> Svc.ContentReplacementService:  # type: ignore
         """Create a new Content Replacement service"""
 
     @json
     @returns.json(Svc.ContentReplacementService)
     @put("services/content-replacement/{service_id}")
-    def update(self, service_id: int, service: Body(type=Svc.ContentReplacementServiceIn)) -> Svc.ContentReplacementService:  # type: ignore
+    def update(
+        self, service_id: int, service: Body(type=Svc.ContentReplacementServiceIn)
+    ) -> Svc.ContentReplacementService:  # type: ignore
         """Update a Content Replacement service"""
 
     def upsert(
@@ -395,7 +435,7 @@ class ContentReplacementServiceSlotsApi(BpkioSdkConsumer):
     def delete(self, service_id, slot_id):
         """Delete a Content Replacement slot, by ID"""
 
-    @response_handler(return_count)
+    @response_handler(extract_count)
     @params({"offset": 0, "limit": 1})
     @get("services/content-replacement/{service_id}/slots")
     def count(
@@ -448,14 +488,17 @@ class VirtualChannelServiceApi(BpkioSdkConsumer):
     @returns.json(Svc.VirtualChannelService)
     @post("services/virtual-channel")
     def create(
-        self, service: Body(type=Svc.VirtualChannelServiceIn)  # type: ignore
+        self,
+        service: Body(type=Svc.VirtualChannelServiceIn),  # type: ignore
     ) -> Svc.VirtualChannelService:  # type: ignore
         """Create a new Virtual Channel service"""
 
     @json
     @returns.json(Svc.VirtualChannelService)
     @put("services/virtual-channel/{service_id}")
-    def update(self, service_id: int, service: Body(type=Svc.VirtualChannelServiceIn)) -> Svc.VirtualChannelService:  # type: ignore
+    def update(
+        self, service_id: int, service: Body(type=Svc.VirtualChannelServiceIn)
+    ) -> Svc.VirtualChannelService:  # type: ignore
         """Update a Virtual Channel service"""
 
     def upsert(
@@ -544,7 +587,9 @@ class VirtualChannelServiceSlotsApi(BpkioSdkConsumer):
     @returns.json(VirtualChannelSlot)
     @post("services/virtual-channel/{service_id}/slots")
     def create(
-        self, service_id, service: Body(type=VirtualChannelSlotIn)  # type: ignore
+        self,
+        service_id,
+        service: Body(type=VirtualChannelSlotIn),  # type: ignore
     ) -> VirtualChannelSlot:  # type: ignore
         """Create a new Virtual Channel slot"""
 
@@ -552,7 +597,7 @@ class VirtualChannelServiceSlotsApi(BpkioSdkConsumer):
     def delete(self, service_id, slot_id):
         """Delete a Virtual Channel slot, by ID"""
 
-    @response_handler(return_count)
+    @response_handler(extract_count)
     @params({"offset": 0, "limit": 1})
     @get("services/virtual-channel/{service_id}/slots")
     def count(
@@ -603,14 +648,17 @@ class AdInsertionServiceApi(BpkioSdkConsumer):
     @returns.json(Svc.AdInsertionService)
     @post("services/ad-insertion")
     def create(
-        self, service: Body(type=Svc.AdInsertionServiceIn)  # type: ignore
+        self,
+        service: Body(type=Svc.AdInsertionServiceIn),  # type: ignore
     ) -> Svc.AdInsertionService:  # type: ignore
         """Create a new Ad Insertion service"""
 
     @json
     @returns.json(Svc.AdInsertionService)
     @put("services/ad-insertion/{service_id}")
-    def update(self, service_id: int, service: Body(type=Svc.AdInsertionServiceIn)) -> Svc.AdInsertionService:  # type: ignore
+    def update(
+        self, service_id: int, service: Body(type=Svc.AdInsertionServiceIn)
+    ) -> Svc.AdInsertionService:  # type: ignore
         """Update an Ad Insertion service"""
 
     def upsert(
@@ -674,14 +722,17 @@ class AdaptiveStreamingCdnServiceApi(BpkioSdkConsumer):
     @returns.json(Svc.AdaptiveStreamingCdnService)
     @post("services/adaptive-streaming-cdn")
     def create(
-        self, service: Body(type=Svc.AdaptiveStreamingCdnServiceIn)  # type: ignore
+        self,
+        service: Body(type=Svc.AdaptiveStreamingCdnServiceIn),  # type: ignore
     ) -> Svc.AdaptiveStreamingCdnService:  # type: ignore
         """Create a new Adaptive Streaming CDN service"""
 
     @json
     @returns.json(Svc.AdaptiveStreamingCdnService)
     @put("services/adaptive-streaming-cdn/{service_id}")
-    def update(self, service_id: int, service: Body(type=Svc.AdaptiveStreamingCdnServiceIn)) -> Svc.AdaptiveStreamingCdnService:  # type: ignore
+    def update(
+        self, service_id: int, service: Body(type=Svc.AdaptiveStreamingCdnServiceIn)
+    ) -> Svc.AdaptiveStreamingCdnService:  # type: ignore
         """Update an Adaptive Streaming CDN service"""
 
     def upsert(
