@@ -1,3 +1,6 @@
+from bpkio_api.credential_provider import InvalidTenantError, NoTenantSectionError
+
+
 class BroadpeakIoApiError(Exception):
     def __init__(self, status_code, message, reason, url):
         self.status_code = status_code
@@ -36,3 +39,10 @@ class InvalidEndpointError(BroadpeakIoApiError):
         self.message = message
         self.original_message = original_message
         super().__init__(message=message, reason=original_message, url=None, status_code=403)
+
+
+class BroadpeakIoSdkError(Exception):
+    def __init__(self, message):
+        self.message = message
+        super().__init__(message)
+
